import logging
from gptcache import cache
from gptcache.processor.post import temperature_softmax
from gptcache.utils.error import NotInitError
from gptcache.utils.time import time_cal


def adapt(llm_handler, cache_data_convert, update_cache_callback, *args, **kwargs):
    user_temperature = "temperature" in kwargs
    user_top_k = "top_k" in kwargs
    temperature = kwargs.pop("temperature", 0.0)
    chat_cache = kwargs.pop("cache_obj", cache)
    require_object_store = kwargs.pop("require_object_store", False)
    if require_object_store:
        assert chat_cache.data_manager.o, "Object store is required for adapter."
    if not chat_cache.has_init:
        raise NotInitError()
    cache_enable = chat_cache.cache_enable_func(*args, **kwargs)
    context = kwargs.pop("cache_context", {})
    embedding_data = None
    # you want to retry to send the request to chatgpt when the cache is negative
    cache_skip = kwargs.pop("cache_skip", False)
    cache_factor = kwargs.pop("cache_factor", 1.0)
    pre_embedding_data = chat_cache.pre_embedding_func(
        kwargs,
        extra_param=context.get("pre_embedding_func", None),
        prompts=chat_cache.config.prompts,
    )
    if cache_enable:
        embedding_data = time_cal(
            chat_cache.embedding_func,
            func_name="embedding",
            report_func=chat_cache.report.embedding,
        )(pre_embedding_data, extra_param=context.get("embedding_func", None))
    if cache_enable and not cache_skip:
        cache_data_list = time_cal(
            chat_cache.data_manager.search,
            func_name="search",
            report_func=chat_cache.report.search,
        )(
            embedding_data,
            extra_param=context.get("search_func", None),
            top_k=kwargs.pop("top_k", 5) if (user_temperature and not user_top_k) else kwargs.pop("top_k", -1),
        )
        if cache_data_list is None:
            cache_data_list = []
        cache_answers = []
        similarity_threshold = chat_cache.config.similarity_threshold
        min_rank, max_rank = chat_cache.similarity_evaluation.range()
        rank_threshold = (max_rank - min_rank) * similarity_threshold * cache_factor
        rank_threshold = (
            max_rank
            if rank_threshold > max_rank
            else min_rank
            if rank_threshold < min_rank
            else rank_threshold
        )
        for cache_data in cache_data_list:
            ret = chat_cache.data_manager.get_scalar_data(
                cache_data, extra_param=context.get("get_scalar_data", None)
            )
            if ret is None:
                continue

            if "deps" in context and hasattr(ret.question, "deps"):
                eval_query_data = {
                    "question": context["deps"][0]["data"],
                    "embedding": None
                }
                eval_cache_data = {
                    "question": ret.question.deps[0].data,
                    "answer": ret.answers[0].answer,
                    "search_result": cache_data,
                    "embedding": None,
                }
            else:
                eval_query_data = {
                    "question": pre_embedding_data,
                    "embedding": embedding_data,
                }

                eval_cache_data = {
                    "question": ret.question,
                    "answer": ret.answers[0].answer,
                    "search_result": cache_data,
                    "embedding": ret.embedding_data,
                }
            rank = chat_cache.similarity_evaluation.evaluation(
                eval_query_data,
                eval_cache_data,
                extra_param=context.get("evaluation_func", None),
            )
            if rank_threshold <= rank:
                cache_answers.append((rank, ret.answers[0].answer))
                chat_cache.data_manager.hit_cache_callback(cache_data)
        cache_answers = sorted(cache_answers, key=lambda x: x[0], reverse=True)
        if len(cache_answers) != 0:
            if chat_cache.post_process_messages_func is temperature_softmax:
                return_message = chat_cache.post_process_messages_func(
                    messages=[t[1] for t in cache_answers],
                    scores=[t[0] for t in cache_answers],
                    temperature=temperature
                )
            else:
                return_message = chat_cache.post_process_messages_func(
                    [t[1] for t in cache_answers]
                )
            chat_cache.report.hint_cache()
            return cache_data_convert(return_message)

    next_cache = chat_cache.next_cache
    if next_cache:
        kwargs["cache_obj"] = next_cache
        kwargs["cache_context"] = context
        kwargs["cache_skip"] = cache_skip
        kwargs["cache_factor"] = cache_factor
        llm_data = adapt(
            llm_handler, cache_data_convert, update_cache_callback, *args, **kwargs
        )
    else:
        llm_data = llm_handler(*args, **kwargs)

    if cache_enable:
        try:

            def update_cache_func(handled_llm_data, question=None):
                if question is None:
                    question = pre_embedding_data
                else:
                    question.content = pre_embedding_data
                chat_cache.data_manager.save(
                    question,
                    handled_llm_data,
                    embedding_data,
                    extra_param=context.get("save_func", None),
                )

            llm_data = update_cache_callback(llm_data, update_cache_func, *args, **kwargs)
        except Exception as e:  # pylint: disable=W0703
            logging.warning("failed to save the data to cache, error: %s", e)
    return llm_data



import base64
from io import BytesIO

from gptcache.adapter.adapter import adapt
from gptcache.manager.scalar_data.base import Answer, DataType
from gptcache.utils.error import CacheError
from gptcache.utils import (
    import_pillow, import_diffusers, import_huggingface
    )

import_pillow()
import_huggingface()
import_diffusers()

from PIL import Image  # pylint: disable=C0413
import diffusers  # pylint: disable=C0413


class StableDiffusionPipeline(diffusers.StableDiffusionPipeline):
    """Diffuser StableDiffusionPipeline Wrapper

    Example:
        .. code-block:: python

            import torch

            from gptcache import cache
            from gptcache.processor.pre import get_prompt
            from gptcache.adapter.diffusers import StableDiffusionPipeline

            # init gptcache
            cache.init(pre_embedding_func=get_prompt)

            # run with gptcache
            model_id = "stabilityai/stable-diffusion-2-1"
            pipe = StableDiffusionPipeline.from_pretrained(model_id, torch_dtype=torch.float16)
            pipe.scheduler = DPMSolverMultistepScheduler.from_config(pipe.scheduler.config)
            pipe = pipe.to("cuda")

            prompt = "a photo of an astronaut riding a horse on mars"
            image = pipe(prompt=prompt).images[0]
    """

    def llm_handler(self, *llm_args, **llm_kwargs):
        try:
            return super().__call__(*llm_args, **llm_kwargs)
        except Exception as e:
            raise CacheError("diffuser error") from e

    def __call__(self, *args, **kwargs):
        def cache_data_convert(cache_data):
            return construct_resp_from_cache(cache_data)

        def update_cache_callback(llm_data, update_cache_func, *args, **kwargs):  # pylint: disable=unused-argument
            img = llm_data["images"][0]
            buffered = BytesIO()
            img.save(buffered, format="PNG")
            img_b64 = base64.b64encode(buffered.getvalue())
            update_cache_func(Answer(img_b64, DataType.IMAGE_BASE64))
            return llm_data

        return adapt(
            self.llm_handler, cache_data_convert, update_cache_callback, *args, **kwargs
        )


def construct_resp_from_cache(img_64):
    im_bytes = base64.b64decode(img_64)   # im_bytes is a binary image
    im_file = BytesIO(im_bytes)  # convert image to file-like object
    img = Image.open(im_file)
    return diffusers.pipelines.stable_diffusion.StableDiffusionPipelineOutput(images=[img], nsfw_content_detected=None)




class Report:
    """Get GPTCache report including time and counts for different operations."""

    def __init__(self):
        self.embedding_all_time = 0
        self.embedding_count = 0
        self.search_all_time = 0
        self.search_count = 0
        self.hint_cache_count = 0

    def embedding(self, delta_time):
        """Embedding counts and time.

        :param delta_time: additional runtime.
        """
        self.embedding_all_time += delta_time
        self.embedding_count += 1

    def search(self, delta_time):
        """Search counts and time.

        :param delta_time: additional runtime.
        """
        self.search_all_time += delta_time
        self.search_count += 1

    def average_embedding_time(self):
        """Average embedding time."""
        return round(
            self.embedding_all_time / self.embedding_count
            if self.embedding_count != 0
            else 0,
            4,
        )

    def average_search_time(self):
        return round(
            self.search_all_time / self.search_count
            if self.embedding_count != 0
            else 0,
            4,
        )

    def hint_cache(self):
        self.hint_cache_count += 1



from typing import Dict, List, Tuple, Any
import numpy as np
from gptcache.utils import (
    import_onnxruntime,
    import_huggingface_hub,
    import_huggingface,
)
from gptcache.similarity_evaluation import SimilarityEvaluation

import_onnxruntime()
import_huggingface_hub()
import_huggingface()

from transformers import AutoTokenizer  # pylint: disable=C0413
from huggingface_hub import hf_hub_download  # pylint: disable=C0413
import onnxruntime  # pylint: disable=C0413


def pad_sequence(input_ids_list: List[np.ndarray], padding_value: int = 0):
    max_len = max(len(sequence) for sequence in input_ids_list)
    padded_sequences = np.full((len(input_ids_list), max_len), padding_value)
    for i, sequence in enumerate(input_ids_list):
        padded_sequences[i, : len(sequence)] = sequence
    return padded_sequences


class OnnxModelEvaluation(SimilarityEvaluation):
    """Using ONNX model to evaluate sentences pair similarity.

    This evaluator use the ONNX model to evaluate the similarity of two sentences.

    :param model: model name of OnnxModelEvaluation. Default is 'GPTCache/albert-duplicate-onnx'.
    :type model: str

    Example:
        .. code-block:: python

            from gptcache.similarity_evaluation import OnnxModelEvaluation

            evaluation = OnnxModelEvaluation()
            score = evaluation.evaluation(
                {
                    'question': 'What is the color of sky?'
                },
                {
                    'question': 'hello'
                }
            )
    """

    def __init__(self, model: str = "GPTCache/albert-duplicate-onnx"):
        tokenizer_name = "albert-base-v2"
        self.tokenizer = AutoTokenizer.from_pretrained(tokenizer_name)
        self.model = model
        onnx_model_path = hf_hub_download(repo_id=model, filename="model.onnx")
        self.ort_session = onnxruntime.InferenceSession(onnx_model_path)

    # WARNING: the model cannot evaluate text with more than 512 tokens
    def evaluation(
        self, src_dict: Dict[str, Any], cache_dict: Dict[str, Any], **_
    ) -> float:
        """Evaluate the similarity score of pair.

        :param src_dict: the query dictionary to evaluate with cache.
        :type src_dict: Dict
        :param cache_dict: the cache dictionary.
        :type cache_dict: Dict

        :return: evaluation score.
        """
        try:
            src_question = src_dict["question"]
            cache_question = cache_dict["question"]
            if src_question.lower() == cache_question.lower():
                return 1
            return self.inference(src_question, [cache_question])
        except Exception:  # pylint: disable=W0703
            return 0

    def range(self) -> Tuple[float, float]:
        """Range of similarity score.

        :return: minimum and maximum of similarity score.
        """
        return 0.0, 1.0

    def inference(self, reference: str, candidates: List[str]) -> np.ndarray:
        """Inference the ONNX model.

        :param reference: reference sentence.
        :type reference: str
        :param candidates: candidate sentences.
        :type candidates: List[str]

        :return: probability score indcates how much is reference similar to candidates.
        """
        n_candidates = len(candidates)
        inference_texts = [
            {"text_a": reference, "text_b": candidate} for candidate in candidates
        ]
        batch_encoding_list = [
            self.tokenizer.encode_plus(
                text["text_a"], text["text_b"], padding="longest"
            )
            for text in inference_texts
        ]

        input_ids_list = [np.array(encode.input_ids) for encode in batch_encoding_list]
        attention_mask_list = [
            np.array(encode.attention_mask) for encode in batch_encoding_list
        ]
        token_type_ids_list = [
            np.array(encode.token_type_ids) for encode in batch_encoding_list
        ]

        padded_input_ids = pad_sequence(
            input_ids_list, padding_value=self.tokenizer.pad_token_id
        )
        padded_attention_mask = pad_sequence(
            attention_mask_list, padding_value=self.tokenizer.pad_token_id
        )
        padded_token_type_ids = pad_sequence(
            token_type_ids_list, padding_value=self.tokenizer.pad_token_id
        )

        ort_inputs = {
            "input_ids": padded_input_ids.reshape(n_candidates, -1),
            "attention_mask": padded_attention_mask.reshape(n_candidates, -1),
            "token_type_ids": padded_token_type_ids.reshape(n_candidates, -1),
        }
        ort_outputs = self.ort_session.run(None, ort_inputs)
        scores = ort_outputs[0][:, 1]
        return scores



import base64
import requests


def get_message_from_openai_answer(openai_resp):
    return openai_resp["choices"][0]["message"]["content"]


def get_stream_message_from_openai_answer(openai_data):
    return openai_data["choices"][0]["delta"].get("content", "")


def get_text_from_openai_answer(openai_resp):
    return openai_resp["choices"][0]["text"]


def get_image_from_openai_b64(openai_resp):
    return openai_resp["data"][0]["b64_json"]


def get_image_from_openai_url(openai_resp):
    url = openai_resp["data"][0]["url"]
    img_content = requests.get(url).content
    img_data = base64.b64encode(img_content)
    return img_data


def get_image_from_path(openai_resp):
    img_path = openai_resp["data"][0]["url"]
    with open(img_path, "rb") as f:
        img_data = base64.b64encode(f.read())
    return img_data


def get_audio_text_from_openai_answer(openai_resp):
    return openai_resp["text"]



import random
from typing import List, Any
import numpy


def random_one(messages: List[Any]) -> Any:
    """Randomly select one result after evaluation.

    :param messages: A list of candidate outputs.
    :type messages: List[Any]

    Example:
        .. code-block:: python

            from gptcache.processor.post import random_one

            messages = ["message 1", "message 2", "message 3"]
            answer = random_one(messages)
    """
    return random.choice(messages)


def first(messages: List[Any]) -> Any:
    """Get the first result after evaluation.

    :param messages: A list of candidate outputs.
    :type messages: List[Any]

    Example:
        .. code-block:: python

            from gptcache.processor.post import first

            messages = ["message 1", "message 2", "message 3"]
            answer = first(messages)
            assert answer = messages[0]
    """
    return messages[0]


def nop(messages: List[Any]) -> Any:
    """No change after evaluation.

    :param messages: A list of candidate outputs.
    :type messages: List[Any]

    Example:
        .. code-block:: python

            from gptcache.processor.post import nop

            messages = ["message 1", "message 2", "message 3"]
            answer = nop(messages)
            assert answer = messages
    """
    return messages


def temperature_softmax(messages: List[Any], scores: List[float], temperature: float = 0.0) -> Any:
    """Post processing with temperature softmax after evaluation.

    :param messages: A list of candidate outputs.
    :type messages: List[Any]
    :param scores: A list of evaluation scores corresponding to `messages`
    :type scores: List[float]
    :param temperature: A non-negative number of sampling temperature, defaults to 0.
                        A higher temperature makes the output more random.
                        A lower temperature means a more deterministic and confident output.
    :type temperature: float

    Example:
        .. code-block:: python

            from gptcache.processor.post import temperature_softmax

            messages = ["message 1", "message 2", "message 3"]
            scores = [0.9, 0.5, 0.1]
            answer = temperature_softmax(messages, scores, temperature=0.5)
    """
    from gptcache.utils import import_scipy  # pylint: disable=C0415

    import_scipy()

    from scipy.special import softmax  # pylint: disable=C0415

    if temperature > 0:
        scores = softmax([x / temperature for x in scores])
        return numpy.random.choice(messages, size=1, p=scores)[0]
    else:
        m_s = list(zip(messages, scores))
        return sorted(m_s, key=lambda x: x[1], reverse=True)[0][0]



from abc import ABCMeta, abstractmethod
from typing import Any, List


class EvictionBase(metaclass=ABCMeta):
    """
    Eviction base.
    """

    @abstractmethod
    def put(self, objs: List[Any]):
        pass

    @abstractmethod
    def get(self, obj: Any):
        pass

    @property
    @abstractmethod
    def policy(self) -> str:
        pass



import numpy as np

from gptcache.embedding.base import BaseEmbedding
from gptcache.utils import (
    import_onnxruntime,
    import_huggingface_hub,
    import_huggingface,
)

import_huggingface()
import_onnxruntime()
import_huggingface_hub()

from transformers import AutoTokenizer, AutoConfig  # pylint: disable=C0413
from huggingface_hub import hf_hub_download  # pylint: disable=C0413
import onnxruntime  # pylint: disable=C0413


class Onnx(BaseEmbedding):
    """Generate text embedding for given text using ONNX Model.

    Example:
        .. code-block:: python

            from gptcache.embedding import Onnx

            test_sentence = 'Hello, world.'
            encoder = Onnx(model='GPTCache/paraphrase-albert-onnx')
            embed = encoder.to_embeddings(test_sentence)
    """

    def __init__(self, model="GPTCache/paraphrase-albert-onnx"):
        tokenizer_name = "sentence-transformers/paraphrase-albert-small-v2"
        self.tokenizer = AutoTokenizer.from_pretrained(tokenizer_name)
        self.model = model
        onnx_model_path = hf_hub_download(repo_id=model, filename="model.onnx")
        self.ort_session = onnxruntime.InferenceSession(onnx_model_path)
        config = AutoConfig.from_pretrained(
            "sentence-transformers/paraphrase-albert-small-v2"
        )
        self.__dimension = config.hidden_size

    def to_embeddings(self, data, **_):
        """Generate embedding given text input.

        :param data: text in string.
        :type data: str

        :return: a text embedding in shape of (dim,).
        """
        encoded_text = self.tokenizer.encode_plus(data, padding="max_length")
        ort_inputs = {
            "input_ids": np.array(encoded_text["input_ids"]).reshape(1, -1),
            "attention_mask": np.array(encoded_text["attention_mask"]).reshape(1, -1),
            "token_type_ids": np.array(encoded_text["token_type_ids"]).reshape(1, -1),
        }

        ort_outputs = self.ort_session.run(None, ort_inputs)
        ort_feat = ort_outputs[0]
        emb = self.post_proc(ort_feat, ort_inputs["attention_mask"])
        return emb.flatten()

    def post_proc(self, token_embeddings, attention_mask):
        input_mask_expanded = (
            np.expand_dims(attention_mask, -1)
            .repeat(token_embeddings.shape[-1], -1)
            .astype(float)
        )
        sentence_embs = np.sum(token_embeddings * input_mask_expanded, 1) / np.maximum(
            input_mask_expanded.sum(1), 1e-9
        )
        return sentence_embs

    @property
    def dimension(self):
        """Embedding dimension.

        :return: embedding dimension
        """
        return self.__dimension



__all__ = ["CacheBase"]

from gptcache.utils.lazy_import import LazyImport

scalar_manager = LazyImport(
    "scalar_manager", globals(), "gptcache.manager.scalar_data.manager"
)


def CacheBase(name: str, **kwargs):
    """Generate specific CacheStorage with the configuration. For example, setting for
       `SQLDataBase` (with `name`, `sql_url` and `table_name` params) to manage SQLite, PostgreSQL, MySQL, MariaDB, SQL Server and Oracle.

    :param name: the name of the cache storage, it is support 'sqlite', 'postgresql', 'mysql', 'mariadb', 'sqlserver' and  'oracle' now.
    :type name: str
    :param sql_url: the url of the sql database for cache, such as '<db_type>+<db_driver>://<username>:<password>@<host>:<port>/<database>',
                    and the default value is related to the `cache_store` parameter, 'sqlite:///./sqlite.db' for 'sqlite',
                    'postgresql+psycopg2://postgres:123456@127.0.0.1:5432/postgres' for 'postgresql',
                    'mysql+pymysql://root:123456@127.0.0.1:3306/mysql' for 'mysql',
                    'mariadb+pymysql://root:123456@127.0.0.1:3307/mysql' for 'mariadb',
                    'mssql+pyodbc://sa:Strongpsw_123@127.0.0.1:1434/msdb?driver=ODBC+Driver+17+for+SQL+Server' for 'sqlserver',
                    'oracle+cx_oracle://oracle:123456@127.0.0.1:1521/?service_name=helowin&encoding=UTF-8&nencoding=UTF-8' for 'oracle'.
    :type sql_url: str
    :param table_name: the table name for sql database, defaults to 'gptcache'.
    :type table_name: str

    :return: CacheStorage.

    Example:
        .. code-block:: python

            from gptcache.manager import CacheBase

            cache_base = CacheBase('sqlite')
    """
    return scalar_manager.CacheBase.get(name, **kwargs)



import os
from typing import List

import numpy as np

from gptcache.manager.vector_data.base import VectorBase, VectorData
from gptcache.utils import import_hnswlib

import_hnswlib()

import hnswlib  # pylint: disable=C0413


class Hnswlib(VectorBase):
    """vector store: hnswlib

    :param index_path: the path to hnswlib index, defaults to 'hnswlib_index.bin'.
    :type index_path: str
    :param dimension: the dimension of the vector, defaults to 0.
    :type dimension: int
    :param top_k: the umber of the vectors results to return, defaults to 1.
    :type top_k: int
    :param max_elements: max_elements of hnswlib, defaults 100000.
    :type max_elements: int
    """

    def __init__(self, index_file_path: str, dimension: int, top_k: int, max_elements: int):
        self._index_file_path = index_file_path
        self._dimension = dimension
        self._max_elements = max_elements
        self._index = hnswlib.Index(space="l2", dim=self._dimension)
        self._top_k = top_k
        if os.path.isfile(self._index_file_path):
            self._index.load_index(self._index_file_path, max_elements=max_elements)
        else:
            self._index.init_index(max_elements=max_elements, ef_construction=100, M=16)
            self._index.set_ef(self._top_k * 2)

    def add(self, key: int, data: np.ndarray):
        np_data = np.array(data).astype("float32").reshape(1, -1)
        self._index.add_items(np_data, np.array([key]))

    def mul_add(self, datas: List[VectorData]):
        data_array, id_array = map(list, zip(*((data.data, data.id) for data in datas)))
        np_data = np.array(data_array).astype("float32")
        ids = np.array(id_array)
        self._index.add_items(np_data, ids)

    def search(self, data: np.ndarray, top_k: int = -1):
        np_data = np.array(data).astype("float32").reshape(1, -1)
        if top_k == -1:
            top_k = self._top_k
        ids, dist = self._index.knn_query(data=np_data, k=top_k)
        return list(zip(dist[0], ids[0]))

    def rebuild(self, ids):
        all_data = self._index.get_items(ids)
        new_index = hnswlib.Index(space="l2", dim=self._dimension)
        new_index.init_index(max_elements=self._max_elements, ef_construction=100, M=16)
        new_index.set_ef(self._top_k * 2)
        self._index = new_index
        datas = []
        for key, data in zip(ids, all_data):
            datas.append(VectorData(id=key, data=data))
        self.mul_add(datas)

    def delete(self, ids):
        for i in ids:
            self._index.mark_deleted(i)

    def flush(self):
        self._index.save_index(self._index_file_path)

    def close(self):
        self.flush()



class CacheError(Exception):
    """GPTCache base error"""


class NotInitError(CacheError):
    """Raise when the cache has been used before it's inited"""
    def __init__(self):
        super().__init__("The cache should be inited before using")


class NotFoundError(CacheError):
    """Raise when getting an unsupported store."""
    def __init__(self, store_type, current_type_name):
        super().__init__(f"Unsupported ${store_type}: {current_type_name}")


class ParamError(CacheError):
    """Raise when receiving an invalid param."""


class PipInstallError(CacheError):
    """Raise when failed to install package."""
    def __init__(self, package):
        super().__init__(f"Ran into error installing {package}.")



# pylint: disable=import-outside-toplevel
from typing import Callable, List, Any

from gptcache.utils.error import NotFoundError


class EvictionBase:
    """
    EvictionBase to evict the cache data.
    """

    def __init__(self):
        raise EnvironmentError(
            "EvictionBase is designed to be instantiated, "
            "please using the `EvictionBase.get(name, policy, maxsize, clean_size)`."
        )

    @staticmethod
    def get(name: str, policy: str, maxsize: int, clean_size: int, on_evict: Callable[[List[Any]], None], **kwargs):
        if name in "memory":
            from gptcache.manager.eviction.memory_cache import MemoryCacheEviction

            eviction_base = MemoryCacheEviction(policy, maxsize, clean_size, on_evict, **kwargs)
        else:
            raise NotFoundError("eviction base", name)
        return eviction_base



__all__ = ["VectorBase"]

from gptcache.utils.lazy_import import LazyImport

vector_manager = LazyImport(
    "vector_manager", globals(), "gptcache.manager.vector_data.manager"
)


def VectorBase(name: str, **kwargs):
    """Generate specific VectorBase with the configuration. For example, setting for
       `Milvus` (with , `host`, `port`, `password`, `secure`, `collection_name`, `index_params`, `search_params`, `local_mode`, `local_data` params),
       `Faiss` (with , `index_path`, `dimension`, `top_k` params),
       `Chromadb` (with `top_k`, `client_settings`, `persist_directory`, `collection_name` params),
       `Hnswlib` (with `index_file_path`, `dimension`, `top_k`, `max_elements` params).

    :param name: the name of the vectorbase, it is support 'milvus', 'faiss', 'chromadb', 'hnswlib' now.
    :type name: str

    :param top_k: the umber of the vectors results to return, defaults to 1.
    :type top_k: int

    :param dimension: the dimension of the vector, defaults to 0.
    :type dimension: int

    :param index_path: the path to Faiss index, defaults to 'faiss.index'.
    :type index_path: str

    :param host: the host for Milvus vector database, defaults to 'localhost'.
    :type host: str
    :param port: the port for Milvus vector database, defaults to '19530'.
    :type port: str
    :param user: the user for Zilliz Cloud, defaults to "".
    :type user: str
    :param password: the password for Zilliz Cloud, defaults to "".
    :type password: str
    :param secure: whether it is https with Zilliz Cloud, defaults to False.
    :type secures: bool
    :param index_params: the index parameters for Milvus, defaults to the HNSW index: {'metric_type': 'L2', 'index_type': 'HNSW', 'params': {'M':
                         8, 'efConstruction': 64}}.
    :type index_params: dict
    :param search_params: the index parameters for Milvus, defaults to None.
    :type search_params: dict
    :param collection_name: the name of the collection for Milvus vector database, defaults to 'gptcache'.
    :type collection_name: str
    :param local_mode: if true, will start a local milvus server.
    :type local_mode: bool
    :param local_data: required when local_mode is True.
    :type local_data: str

    :param client_settings: the setting for Chromadb.
    :type client_settings: Settings
    :param persist_directory: the directory to persist, defaults to '.chromadb/' in the current directory.
    :type persist_directory: str

    :param index_path: the path to hnswlib index, defaults to 'hnswlib_index.bin'.
    :type index_path: str
    :param max_elements: max_elements of hnswlib, defaults 100000.
    :type max_elements: int
    """
    return vector_manager.VectorBase.get(name, **kwargs)



from gptcache.manager.scalar_data import CacheBase
from gptcache.manager.vector_data import VectorBase
from gptcache.manager.object_data import ObjectBase
from gptcache.manager.factory import get_data_manager, manager_factory



from gptcache.utils.error import NotFoundError


class ObjectBase:
    """
    ObjectBase to manager the object storage.
    """

    def __init__(self):
        raise EnvironmentError(
            "CacheBase is designed to be instantiated, please using the `CacheBase.get(name)`."
        )

    @staticmethod
    def get(name, **kwargs):
        if name == "local":
            from gptcache.manager.object_data.local_storage import LocalObjectStorage  # pylint: disable=import-outside-toplevel
            object_base = LocalObjectStorage(kwargs.get("path", "./local_obj"))
        elif name == "s3":
            from gptcache.manager.object_data.s3_storage import S3Storage  # pylint: disable=import-outside-toplevel
            object_base = S3Storage(kwargs.get("path_prefix"), kwargs.get("bucket"),
                                    kwargs.get("access_key"), kwargs.get("secret_key"),
                                    kwargs.get("endpoint"))
        else:
            raise NotFoundError("object store", name)
        return object_base



from typing import Optional, Callable, List

from gptcache.utils.error import CacheError


class Config:
    """Pass configuration.

    :param log_time_func: optional, customized log time function
    :param similarity_threshold: a threshold ranged from 0 to 1 to filter search results with similarity score higher than the threshold.
                                 When it is 0, there is no hits. When it is 1, all search results will be returned as hits.
    :type similarity_threshold: float

    Example:
        .. code-block:: python

            from gptcache import Config

            configs = Config(similarity_threshold=0.6)
    """

    def __init__(
            self,
            log_time_func: Optional[Callable[[str, float], None]] = None,
            similarity_threshold: float = 0.8,
            prompts: Optional[List[str]] = None
    ):
        if similarity_threshold < 0 or similarity_threshold > 1:
            raise CacheError(
                "Invalid the similarity threshold param, reasonable range: 0-1"
            )
        self.log_time_func = log_time_func
        self.similarity_threshold = similarity_threshold
        self.prompts = prompts



import os
from typing import Union, Callable
from pathlib import Path
from gptcache.manager.data_manager import SSDataManager, MapDataManager
from gptcache.manager import CacheBase, VectorBase, ObjectBase


def manager_factory(manager="map",
                    data_dir="./",
                    max_size=1000,
                    clean_size=None,
                    eviction: str = "LRU",
                    get_data_container: Callable = None,
                    scalar_params=None,
                    vector_params=None,
                    object_params=None):

    """Factory of DataManager.
       By using this factory method, you only need to specify the root directory of the data,
       and it can automatically manage all the local files.

    :param manager: Type of DataManager. Supports: Map, or {scalar_name},{vector_name}
                    or {scalar_name},{vector_name},{object_name}
    :type manager: str
    :param data_dir: Root path for data storage.
    :type data_dir: str
    :param max_size: the max size for the cache, defaults to 1000.
    :type max_size: int
    :param clean_size: the size to clean up, defaults to `max_size * 0.2`.
    :type clean_size: int
    :param eviction: the eviction policy, it is support "LRU" and "FIFO" now, and defaults to "LRU".
    :type eviction:  str
    :param get_data_container: a Callable to get the data container, defaults to None.
    :type get_data_container:  Callable

    :param scalar_params: Params of scalar storage.
    :type scalar_params:  dict

    :param vector_params: Params of vector storage.
    :type vector_params:  dict

    :param object_params: Params of object storage.
    :type object_params:  dict

    :return: SSDataManager or MapDataManager.

    Example:
        .. code-block:: python

            from gptcache.manager import manager_factory

            data_manager = manager_factory("sqlite,faiss", data_dir="./workspace", vector_params={"dimension": 128})
    """

    Path(data_dir).mkdir(parents=True, exist_ok=True)
    manager = manager.lower()

    if manager == "map":
        return MapDataManager(os.path.join(data_dir, "data_map.txt"), max_size, get_data_container)

    db_infos = manager.split(",")
    if len(db_infos) not in [2, 3]:
        raise RuntimeError("Error manager format: %s, the correct is \"{scalar},{vector},{object}\", object is optional" % manager)

    if len(db_infos) == 2:
        db_infos.append("")
    scalar, vector, obj = db_infos

    if scalar_params is None:
        scalar_params = {}
    if scalar == "sqlite":
        scalar_params["sql_url"] = "sqlite:///" + os.path.join(data_dir, "sqlite.db")
    s = CacheBase(name=scalar, **scalar_params)

    if vector_params is None:
        vector_params = {}
    if vector == "faiss":
        vector_params["index_path"] = os.path.join(data_dir, "faiss.index")
    elif vector == "hnswlib":
        vector_params["index_path"] = os.path.join(data_dir, "hnswlib.index")
    elif vector == "milvus" and vector_params.get("local_mode", False) is True:
        vector_params["local_data"] = os.path.join(data_dir, "milvus_data")
    v = VectorBase(name=vector, **vector_params)

    if object_params is None:
        object_params = {}
    if obj == "local":
        object_params["path"] = os.path.join(data_dir, "local_obj")
    o = ObjectBase(name=obj, **object_params) if obj else None
    return get_data_manager(s, v, o, max_size, clean_size, eviction)


def get_data_manager(
    cache_base: Union[CacheBase, str] = None,
    vector_base: Union[VectorBase, str] = None,
    object_base: Union[ObjectBase, str] = None,
    max_size: int = 1000,
    clean_size: int = None,
    eviction: str = "LRU",
    data_path: str = "data_map.txt",
    get_data_container: Callable = None,
):
    """Generate `SSDataManager` (with `cache_base`, `vector_base`, `max_size`, `clean_size` and `eviction` params),
       or `MAPDataManager` (with `data_path`, `max_size` and `get_data_container` params) to manager the data.

    :param cache_base: a CacheBase object, or the name of the cache storage, it is support 'sqlite', 'postgresql',
                       'mysql', 'mariadb', 'sqlserver' and  'oracle' now.
    :type cache_base: :class:`CacheBase` or str
    :param vector_base: a VectorBase object, or the name of the vector storage, it is support 'milvus', 'faiss' and
                        'chromadb' now.
    :type vector_base: :class:`VectorBase` or str
    :param object_base: a object storage, supports local path and s3.
    :type object_base: :class:`ObjectBase` or str
    :param max_size: the max size for the cache, defaults to 1000.
    :type max_size: int
    :param clean_size: the size to clean up, defaults to `max_size * 0.2`.
    :type clean_size: int
    :param eviction: the eviction policy, it is support "LRU" and "FIFO" now, and defaults to "LRU".
    :type eviction:  str
    :param data_path: the path to save the map data, defaults to 'data_map.txt'.
    :type data_path:  str
    :param get_data_container: a Callable to get the data container, defaults to None.
    :type get_data_container:  Callable


    :return: SSDataManager or MapDataManager.

    Example:
        .. code-block:: python

            from gptcache.manager import get_data_manager, CacheBase, VectorBase

            data_manager = get_data_manager(CacheBase('sqlite'), VectorBase('faiss', dimension=128))
    """
    if not cache_base and not vector_base:
        return MapDataManager(data_path, max_size, get_data_container)

    if isinstance(cache_base, str):
        cache_base = CacheBase(name=cache_base)
    if isinstance(vector_base, str):
        vector_base = VectorBase(name=vector_base)
    if isinstance(object_base, str):
        object_base = ObjectBase(name=object_base)
    assert cache_base and vector_base
    return SSDataManager(cache_base, vector_base, object_base, max_size, clean_size, eviction)



from abc import ABC, abstractmethod
from typing import Any, List


class ObjectBase(ABC):
    """
    Object storage base.
    """

    @abstractmethod
    def put(self, obj: Any) -> str:
        pass

    @abstractmethod
    def get(self, obj: str) -> Any:
        pass

    @abstractmethod
    def get_access_link(self, obj: str) -> str:
        pass

    @abstractmethod
    def delete(self, to_delete: List[str]):
        pass



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import List

import numpy as np


@dataclass
class VectorData:
    id: int
    data: np.ndarray


class VectorBase(ABC):
    """VectorBase: base vector store interface"""

    @abstractmethod
    def mul_add(self, datas: List[VectorData]):
        pass

    @abstractmethod
    def search(self, data: np.ndarray, top_k: int):
        pass

    @abstractmethod
    def rebuild(self, ids=None) -> bool:
        pass

    @abstractmethod
    def delete(self, ids) -> bool:
        pass

    def flush(self):
        pass

    def close(self):
        pass



from typing import Optional, List, Any

from gptcache.adapter.adapter import adapt
from gptcache.utils import import_pydantic, import_langchain
from gptcache.manager.scalar_data.base import Answer, DataType

import_pydantic()
import_langchain()

# pylint: disable=C0413
from pydantic import BaseModel
from langchain.llms.base import LLM
from langchain.chat_models.base import BaseChatModel
from langchain.schema import BaseMessage, LLMResult, AIMessage, ChatGeneration, ChatResult


class LangChainLLMs(LLM, BaseModel):
    """LangChain LLM Wrapper.

    :param llm: LLM from langchain.llms.
    :type llm: Any

    Example:
        .. code-block:: python

            from gptcache import cache
            from gptcache.processor.pre import get_prompt
            # init gptcache
            cache.init(pre_embedding_func=get_prompt)
            cache.set_openai_key()

            from langchain.llms import OpenAI
            from gptcache.adapter.langchain_models import LangChainLLMs
            # run llm with gptcache
            llm = LangChainLLMs(llm=OpenAI(temperature=0))
            llm("Hello world")
    """

    llm: Any

    @property
    def _llm_type(self) -> str:
        return "gptcache_llm"

    def _call(self, prompt: str, stop: Optional[List[str]] = None, **kwargs) -> str:
        return adapt(
            self.llm,
            cache_data_convert,
            update_cache_callback,
            prompt=prompt,
            stop=stop,
            **kwargs
        )

    def __call__(self, prompt: str, stop: Optional[List[str]] = None, **kwargs) -> str:
        return self._call(prompt=prompt, stop=stop, **kwargs)


# pylint: disable=protected-access
class LangChainChat(BaseChatModel, BaseModel):
    """LangChain LLM Wrapper.

    :param chat: LLM from langchain.chat_models.
    :type chat: Any

    Example:
        .. code-block:: python

            from gptcache import cache
            from gptcache.processor.pre import get_prompt
            # init gptcache
            cache.init(pre_embedding_func=get_prompt)
            cache.set_openai_key()
            from langchain.chat_models import ChatOpenAI
            from gptcache.adapter.langchain_models import LangChainChat
            # run chat model with gptcache
            chat = LangChainLLMs(chat=ChatOpenAI(temperature=0))
            chat([HumanMessage(content="Translate this sentence from English to French. I love programming.")])
    """

    chat: Any

    def _generate(self, messages: Any, stop: Optional[List[str]] = None, **kwargs):
        return adapt(
            self.chat._generate,
            cache_msg_data_convert,
            update_cache_msg_callback,
            messages=messages,
            stop=stop,
            **kwargs
        )

    async def _agenerate(self, messages: List[List[BaseMessage]], stop: Optional[List[str]] = None, **kwargs) -> LLMResult:
        return adapt(
            self.chat._agenerate,
            cache_msg_data_convert,
            update_cache_msg_callback,
            messages=messages,
            stop=stop,
            **kwargs
        )

    def __call__(self, messages: Any, stop: Optional[List[str]] = None, **kwargs):
        res = self._generate(messages=messages, stop=stop, **kwargs)
        return res.generations[0].message


def cache_data_convert(cache_data):
    return cache_data


def update_cache_callback(llm_data, update_cache_func, *args, **kwargs):  # pylint: disable=unused-argument
    update_cache_func(Answer(llm_data, DataType.STR))
    return llm_data


def cache_msg_data_convert(cache_data):
    llm_res = ChatResult(generations=[ChatGeneration(text="",
                                                     generation_info=None,
                                                     message=AIMessage(content=cache_data, additional_kwargs={}))],
                         llm_output=None)
    return llm_res


def update_cache_msg_callback(llm_data, update_cache_func, *args, **kwargs):  # pylint: disable=unused-argument
    update_cache_func(llm_data.generations[0].text)
    return llm_data



from typing import Any, Optional, Callable, List

from gptcache import Cache, cache, Config
from gptcache.adapter.adapter import adapt
from gptcache.embedding import Onnx
from gptcache.manager import manager_factory
from gptcache.processor.post import first
from gptcache.processor.pre import get_prompt
from gptcache.similarity_evaluation import SearchDistanceEvaluation


def _cache_data_converter(cache_data):
    """For cache results, do nothing"""
    return cache_data


def _update_cache_callback_none(
    llm_data, update_cache_func, *args, **kwargs  # pylint: disable=W0613
) -> None:
    """When updating cached data, do nothing, because currently only cached queries are processed"""
    return None


def _llm_handle_none(*llm_args, **llm_kwargs) -> None:  # pylint: disable=W0613
    """Do nothing on a cache miss"""
    return None


def _update_cache_callback(
    llm_data, update_cache_func, *args, **kwargs
):  # pylint: disable=W0613
    """Save the `llm_data` to cache storage"""
    update_cache_func(llm_data)


def put(prompt: str, data: Any, **kwargs) -> None:
    """save api, save qa pair information to GPTCache
    Please make sure that the `pre_embedding_func` param is `get_prompt` when initializing the cache

    Example:
        .. code-block:: python

            from gptcache.adapter.api import put
            from gptcache.processor.pre import get_prompt

            cache.init(pre_embedding_func=get_prompt)
            put("hello", "foo")
    """

    def llm_handle(*llm_args, **llm_kwargs):  # pylint: disable=W0613
        return data

    adapt(
        llm_handle,
        _cache_data_converter,
        _update_cache_callback,
        cache_skip=True,
        prompt=prompt,
        **kwargs,
    )


def get(prompt: str, **kwargs) -> Any:
    """search api, search the cache data according to the `prompt`
    Please make sure that the `pre_embedding_func` param is `get_prompt` when initializing the cache

    Example:
        .. code-block:: python

            from gptcache.adapter.api import put, get
            from gptcache.processor.pre import get_prompt

            cache.init(pre_embedding_func=get_prompt)
            put("hello", "foo")
            print(get("hello"))
    """
    res = adapt(
        _llm_handle_none,
        _cache_data_converter,
        _update_cache_callback_none,
        prompt=prompt,
        **kwargs,
    )
    return res


def init_similar_cache(
    data_dir: str = "api_cache",
    cache_obj: Optional[Cache] = None,
    post_func: Callable[[List[Any]], Any] = first,
    config: Config = Config(),
):
    onnx = Onnx()
    data_manager = manager_factory(
        "sqlite,faiss", data_dir=data_dir, vector_params={"dimension": onnx.dimension}
    )
    evaluation = SearchDistanceEvaluation()
    cache_obj = cache_obj if cache_obj else cache
    cache_obj.init(
        pre_embedding_func=get_prompt,
        embedding_func=onnx.to_embeddings,
        data_manager=data_manager,
        similarity_evaluation=evaluation,
        post_process_messages_func=post_func,
        config=config,
    )



from typing import Any, List
import os
import uuid
from pathlib import Path
from gptcache.manager.object_data.base import ObjectBase
from gptcache.utils.log import gptcache_log


class LocalObjectStorage(ObjectBase):
    """Local object storage
    """

    def __init__(self, local_root: str):
        self._local_root = Path(local_root)
        self._local_root.mkdir(exist_ok=True)

    def put(self, obj: Any) -> str:
        f_path = self._local_root / str(uuid.uuid4())
        with open(f_path, "wb") as f:
            f.write(obj)
        return str(f_path.absolute())

    def get(self, obj: str) -> Any:
        try:
            with open(obj, "rb") as f:
                return f.read()
        except Exception: # pylint: disable=broad-except
            return None

    def get_access_link(self, obj: str, _: int = 3600):
        return obj

    def delete(self, to_delete: List[str]):
        assert isinstance(to_delete, list)
        for obj in to_delete:
            try:
                os.remove(obj)
            except Exception:  # pylint: disable=broad-except
                gptcache_log.warning("Can not find obj: %s", obj)
                pass



from gptcache.adapter.adapter import adapt
from gptcache.utils.error import CacheError
from gptcache.manager.scalar_data.base import DataType, Question, Answer

from argparse import Namespace

from minigpt4.common.config import Config
from minigpt4.common.registry import registry
from minigpt4.conversation.conversation import Chat, CONV_VISION

# pylint: disable=wildcard-import
# imports modules for registration
from minigpt4.datasets.builders import *
from minigpt4.models import *
from minigpt4.processors import *
from minigpt4.runners import *
from minigpt4.tasks import *


class MiniGPT4:  # pragma: no cover
    """MiniGPT4 Wrapper

    Example:
        .. code-block:: python
            from gptcache import cache
            from gptcache.processor.pre import get_image_question
            from gptcache.adapter.minigpt4 import MiniGPT4

            # init gptcache
            cache.init(pre_embedding_func=get_image_question)

            # run with gptcache
            pipe = MiniGPT4.from_pretrained(cfg_path='eval_configs/minigpt4_eval.yaml', gpu_id=3, options=None)
            question = "Which city is this photo taken?"
            image = "./merlion.png"
            answer = pipe(image, question)
    """
    def __init__(self, chat, return_hit):
        self.chat = chat
        self.return_hit = return_hit

    @classmethod
    def from_pretrained(cls, cfg_path, gpu_id=0, options=None,  return_hit=False):
        args = Namespace(cfg_path=cfg_path, gpu_id=gpu_id, options=options)
        cfg = Config(args)
        model_config = cfg.model_cfg
        model_config.device_8bit = args.gpu_id
        model_cls = registry.get_model_class(model_config.arch)
        model = model_cls.from_config(model_config).to("cuda:{}".format(args.gpu_id))

        vis_processor_cfg = cfg.datasets_cfg.cc_sbu_align.vis_processor.train
        vis_processor = registry.get_processor_class(vis_processor_cfg.name).from_config(vis_processor_cfg)
        chat = Chat(model, vis_processor, device="cuda:{}".format(args.gpu_id))
        return cls(chat, return_hit)

    def llm_handler(self, image, question):
        chat_state = CONV_VISION.copy()
        img_list = []
        try:
            self.chat.upload_img(image, chat_state, img_list)
            self.chat.ask(question, chat_state)
            answer = self.chat.answer(conv=chat_state, img_list=img_list)[0]
            return answer if not self.return_hit else answer, False
        except Exception as e:
            raise CacheError("minigpt4 error") from e

    def __call__(self, image, question, *args, **kwargs):
        cache_context = {"deps": [
            {"name": "text", "data": question, "dep_type": DataType.STR},
            {"name": "image", "data": image, "dep_type": DataType.STR},
        ]}

        def cache_data_convert(cache_data):
            return cache_data if not self.return_hit else cache_data, True

        def update_cache_callback(llm_data, update_cache_func, *args, **kwargs):  # pylint: disable=unused-argument
            question_data = Question.from_dict({
                "content": "pre_embedding_data",
                "deps": [
                    {"name": "text", "data": kwargs["question"], "dep_type": DataType.STR},
                    {"name": "image", "data": kwargs["image"], "dep_type": DataType.STR},
                ]
            })
            llm_data_cache = llm_data if not self.return_hit else llm_data[0]
            update_cache_func(Answer(llm_data_cache, DataType.STR), question=question_data)
            return llm_data

        return adapt(
            self.llm_handler, cache_data_convert, update_cache_callback, image=image, question=question, cache_context=cache_context, *args, **kwargs
        )



from gptcache.similarity_evaluation.similarity_evaluation import SimilarityEvaluation

__all__ = [
    "SimilarityEvaluation",
    "OnnxModelEvaluation",
    "NumpyNormEvaluation",
    "SearchDistanceEvaluation",
    "ExactMatchEvaluation",
    "KReciprocalEvaluation",
]

from gptcache.utils.lazy_import import LazyImport

onnx = LazyImport("onnx", globals(), "gptcache.similarity_evaluation.onnx")
np = LazyImport("np", globals(), "gptcache.similarity_evaluation.np")
distance = LazyImport("simple", globals(), "gptcache.similarity_evaluation.distance")
exact_match = LazyImport(
    "exact_match", globals(), "gptcache.similarity_evaluation.exact_match"
)
kreciprocal = LazyImport(
    "kreciprocal", globals(), "gptcache.similarity_evaluation.kreciprocal"
)


def OnnxModelEvaluation(model="GPTCache/albert-duplicate-onnx"):
    return onnx.OnnxModelEvaluation(model)


def NumpyNormEvaluation(enable_normal: bool = False, **kwargs):
    return np.NumpyNormEvaluation(enable_normal, **kwargs)


def SearchDistanceEvaluation(max_distance=4.0, positive=False):
    return distance.SearchDistanceEvaluation(max_distance, positive)


def ExactMatchEvaluation():
    return exact_match.ExactMatchEvaluation()


def KReciprocalEvaluation(vectordb, top_k=3, max_distance=4.0, positive=False):
    return kreciprocal.KReciprocalEvaluation(vectordb, top_k, max_distance, positive)



__all__ = ["OpenAI", "Huggingface", "SBERT", "Cohere", "Onnx", "FastText", "Data2VecAudio", "Timm", "ViT"]


from gptcache.utils.lazy_import import LazyImport

openai = LazyImport("openai", globals(), "gptcache.embedding.openai")
huggingface = LazyImport("huggingface", globals(), "gptcache.embedding.huggingface")
sbert = LazyImport("sbert", globals(), "gptcache.embedding.sbert")
onnx = LazyImport("onnx", globals(), "gptcache.embedding.onnx")
cohere = LazyImport("cohere", globals(), "gptcache.embedding.cohere")
fasttext = LazyImport("fasttext", globals(), "gptcache.embedding.fasttext")
data2vec = LazyImport("data2vec", globals(), "gptcache.embedding.data2vec")
timm = LazyImport("timm", globals(), "gptcache.embedding.timm")
vit = LazyImport("vit", globals(), "gptcache.embedding.vit")


def Cohere(model="large", api_key=None):
    return cohere.Cohere(model, api_key)


def OpenAI(model="text-embedding-ada-002", api_key=None):
    return openai.OpenAI(model, api_key)


def Huggingface(model="sentence-transformers/all-mpnet-base-v2"):
    return huggingface.Huggingface(model)


def SBERT(model="all-MiniLM-L6-v2"):
    return sbert.SBERT(model)


def Onnx(model="GPTCache/paraphrase-albert-onnx"):
    return onnx.Onnx(model)


def FastText(model="en", dim=None):
    return fasttext.FastText(model, dim)


def Data2VecAudio(model="facebook/data2vec-audio-base-960h"):
    return data2vec.Data2VecAudio(model)


def Timm(model="resnet50", device="default"):
    return timm.Timm(model, device)


def ViT(model="google/vit-base-patch16-384"):
    return vit.ViT(model)



__all__ = ["ObjectBase"]

from gptcache.utils.lazy_import import LazyImport

object_manager = LazyImport(
    "object_manager", globals(), "gptcache.manager.object_data.manager"
)


def ObjectBase(name: str, **kwargs):
    """Generate specific ObjectStorage with the configuration. For example, setting for
       `ObjectBase` (with `name`) to manage LocalObjectStorage, S3 object storage.

    :param name: the name of the object storage, it is support 'local', 's3'.
    :type name: str
    :param path: the cache root of the LocalObjectStorage.
    :type path: str

    :param bucket: the bucket of s3.
    :type bucket: str
    :param path_prefix: s3 object prefix.
    :type path_prefix: str
    :param access_key: the access_key of s3.
    :type access_key: str
    :param secret_key: the secret_key of s3.
    :type secret_key: str

    :return: ObjectStorage.

    Example:
        .. code-block:: python

            from gptcache.manager import ObjectBase

            obj_storage = ObjectBase('local', path='./')
    """
    return object_manager.ObjectBase.get(name, **kwargs)
